/*************************************************************************************
	NOTEMIDIECHO - command line utility for echoing MIDI input to MIDI output

		(C) 2001 by Michael Minn - michael@michaelminn.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

*************************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
//#include <linux/soundcard.h>

void print_message(unsigned char *buffer, int length)
{
	for (int scan = 0; scan < length; ++scan)
		printf("%02x ", ((unsigned int) buffer[scan]) & 0xff);
	printf("\n");
}

int main (int argc, char **argv)
{
	printf("NOTEMIDIECHO v2001.07.01 by Michael Minn\n");

	int print = 0;
	char *file_name = argv[1];

	if ((argc > 3) || (argc < 2) || ((argc == 3) && (strcmp(argv[1], "-l") != 0)))
	{
		printf("SYNTAX: notemidiecho [-l] <midi or audio device file>\n");
		printf("        echoes MIDI or audio input from a device back to the device.\n");
		printf("        -l option gives hexadecimal echo to the console\n");
		return -1;
	}

	if (argc == 3)
	{
		print = 1;
		file_name = argv[2];
	}

	int descriptor = open(file_name, O_RDWR | O_SYNC);

	if (descriptor < 0)
	{
		printf("Failure opening %s: %s\n", file_name, strerror(errno));
		return -1;
	}

	/***
	if (strstr(file_name, "sequencer"))
	{
		unsigned char timer_start[8] = { EV_TIMING, TMR_START, 0, 0, 0, 0, 0, 0 };
		write(descriptor, timer_start, sizeof(timer_start));
	}
	***/

	int bytes = 0;
	int status = 0;
	unsigned char buffer[256];
	while (status >= 0)
		if ((status = read(descriptor, buffer, sizeof(buffer))) < 0)
			printf("Failure reading %s: %s\n", file_name, strerror(errno));

		else if ((bytes = status) == 0)
			; // busy wait

		else if ((status = write(descriptor, buffer, status)) < 0)
		{
			printf("Failure writing %d bytes to %s: %s\n", bytes, file_name, strerror(errno));
			print_message(buffer, bytes);
		}

		else if (print)
			print_message(buffer, bytes);

	return 0;
}
